"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
const constants_1 = require("../common/constants");
const decorators_1 = require("../common/decorators");
const events_1 = require("events");
class IOSLogParserService extends events_1.EventEmitter {
    constructor($deviceLogProvider, $iosDeviceOperations, $iOSSimulatorLogProvider, $logger, $projectData) {
        super();
        this.$deviceLogProvider = $deviceLogProvider;
        this.$iosDeviceOperations = $iosDeviceOperations;
        this.$iOSSimulatorLogProvider = $iOSSimulatorLogProvider;
        this.$logger = $logger;
        this.$projectData = $projectData;
    }
    startParsingLog(device) {
        this.$deviceLogProvider.setProjectNameForDevice(device.deviceInfo.identifier, this.$projectData.projectName);
        this.startParsingLogCore(device);
        this.startLogProcess(device);
    }
    startParsingLogCore(device) {
        const logProvider = device.isEmulator ? this.$iOSSimulatorLogProvider : this.$iosDeviceOperations;
        logProvider.on(constants_1.DEVICE_LOG_EVENT_NAME, (response) => this.processDeviceLogResponse(response));
    }
    processDeviceLogResponse(response) {
        const matches = IOSLogParserService.MESSAGE_REGEX.exec(response.message);
        if (matches) {
            const data = {
                port: parseInt(matches[1]),
                appId: matches[2],
                deviceId: response.deviceId
            };
            this.$logger.trace(`Emitting ${constants_1.DEBUGGER_PORT_FOUND_EVENT_NAME} event`, data);
            this.emit(constants_1.DEBUGGER_PORT_FOUND_EVENT_NAME, data);
        }
    }
    startLogProcess(device) {
        if (device.isEmulator) {
            return this.$iOSSimulatorLogProvider.startNewMutedLogProcess(device.deviceInfo.identifier);
        }
        return this.$iosDeviceOperations.startDeviceLog(device.deviceInfo.identifier);
    }
}
IOSLogParserService.MESSAGE_REGEX = /NativeScript debugger has opened inspector socket on port (\d+?) for (.*)[.]/;
__decorate([
    decorators_1.cache()
], IOSLogParserService.prototype, "startParsingLogCore", null);
exports.IOSLogParserService = IOSLogParserService;
$injector.register("iOSLogParserService", IOSLogParserService);
